<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

class WPCC_Custom_Cursors_REST_API {

    public function __construct() {
        add_action('rest_api_init', [$this, 'wpcc_register_rest_routes']);
    }

    public function wpcc_register_rest_routes() {

        // Endpoint to save cursor customizations
        register_rest_route('custom/v1', '/cursor-customization', [
            'methods' => 'POST',
            'callback' => [$this, 'wpcc_save_cursor_customization'],
            'permission_callback' => '__return_true',
        ]);

        // Endpoint to fetch cursor customizations
        register_rest_route('custom/v1', '/cursor-customization', [
            'methods' => 'GET',
            'callback' => [$this, 'wpcc_get_cursor_customization'],
            'permission_callback' => '__return_true',
        ]);

        // Endpoint to delete cursor customizations
        register_rest_route('custom/v1', '/cursor-customization', [
            'methods' => 'DELETE',
            'callback' => [$this, 'wpcc_delete_cursor_customization'],
            'permission_callback' => '__return_true',
        ]);

        // Register new REST route for saving cursor placement settings
        register_rest_route('custom/v1', '/cursor-settings', [
            'methods' => 'POST',
            'callback' => [$this, 'wpcc_save_cursor_settings'],
            'permission_callback' => '__return_true',
        ]);

        // For updating existing cursors (new route)
        register_rest_route('custom/v1', '/cursor-settings/(?P<id>\d+)', [
            'methods' => 'PUT',
            'callback' => [$this, 'wpcc_update_cursor_settings'],
            'permission_callback' => '__return_true',
        ]);

        // Register a GET route to fetch all saved cursor settings
        register_rest_route('custom/v1', '/cursor-settings', [
            'methods' => 'GET',
            'callback' => [$this, 'wpcc_get_cursor_settings'],
            'permission_callback' => '__return_true',
        ]);

        // Endpoint to delete a specific cursor setting
        register_rest_route('custom/v1', '/cursor-settings/(?P<cursor_id>\d+)', [
            'methods' => 'DELETE',
            'callback' => [$this, 'wpcc_delete_cursor_settings'],
            'permission_callback' => '__return_true',
        ]);

        // Endpoint to reset all cursor data (new)
        register_rest_route('custom/v1', '/reset-all-cursors', [
            'methods' => 'POST',
            'callback' => [$this, 'wpcc_reset_all_cursors'],
            'permission_callback' => '__return_true', // Adjust permission if needed
        ]);
    }

    public function wpcc_save_cursor_customization(WP_REST_Request $request) {
        $cursor_id = $request->get_param('cursor_id');
        $customization = $request->get_param('customization');

        if ($cursor_id && $customization) {
            // Get all customizations
            $all_customizations = get_option('cursor_customizations', []);

            // Update the specific cursor's customization
            $all_customizations[$cursor_id] = $customization;

            // Save all customizations back to the database
            update_option('cursor_customizations', $all_customizations);

            return rest_ensure_response(['success' => true]);
        }

        return new WP_Error('invalid_request', 'Missing cursor_id or customization data', ['status' => 400]);
    }

    public function wpcc_get_cursor_customization() {
        // Get all customizations
        $all_customizations = get_option('cursor_customizations', []);
        return rest_ensure_response(['customization' => $all_customizations]);
    }

    public function wpcc_delete_cursor_customization(WP_REST_Request $request) {
        $cursor_id = $request->get_param('cursor_id');

        if ($cursor_id) {
            // Get all customizations
            $all_customizations = get_option('cursor_customizations', []);

            // Check if the cursor ID exists in customizations
            if (isset($all_customizations[$cursor_id])) {
                // Remove the specific cursor's customization
                unset($all_customizations[$cursor_id]);

                // Save updated customizations back to the database
                update_option('cursor_customizations', $all_customizations);

                return rest_ensure_response(['success' => true]);
            } else {
                return new WP_Error('not_found', 'Cursor customization not found', ['status' => 404]);
            }
        }

        return new WP_Error('invalid_request', 'Missing cursor_id', ['status' => 400]);
    }

    public function wpcc_save_cursor_settings(WP_REST_Request $request) {
        $cursor = $request->get_param('cursor'); // Full cursor object
        $placement = sanitize_text_field($request->get_param('placement'));
        $section_identifier = sanitize_text_field($request->get_param('section_identifier'));

        if (!$cursor || !$placement) {
            return new WP_Error('invalid_request', 'Missing required fields.', ['status' => 400]);
        }

        // Get existing cursor settings
        $cursor_settings = get_option('cursor_settings', []);

        // Append the new cursor configuration
        $cursor_settings[] = [
            'cursor' => $cursor, // Store the full cursor object
            'placement' => $placement,
            'section_identifier' => $section_identifier,
            'timestamp' => time(), // Store timestamp for debugging/sorting if needed
        ];

        // Save updated cursor settings array back to the database
        update_option('cursor_settings', $cursor_settings);

        return rest_ensure_response(['success' => true, 'data' => $cursor_settings]);
    }

    public function wpcc_get_cursor_settings() {
        $cursor_settings = get_option('cursor_settings', []);
        if (!empty($cursor_settings)) {
            return rest_ensure_response(['success' => true, 'settings' => $cursor_settings]);
        }

        return rest_ensure_response(['success' => false, 'settings' => []]);
    }

    public function wpcc_delete_cursor_settings(WP_REST_Request $request) {
        $cursor_id = $request->get_param('cursor_id');

        if (!$cursor_id) {
            return new WP_Error('invalid_request', 'Missing cursor_id', ['status' => 400]);
        }

        $cursor_settings = get_option('cursor_settings', []);

        // Find and remove the cursor by ID
        foreach ($cursor_settings as $index => $cursor) {
            if (isset($cursor['cursor']['id']) && $cursor['cursor']['id'] == $cursor_id) {
                unset($cursor_settings[$index]);
                update_option('cursor_settings', array_values($cursor_settings)); // Re-index the array
                return rest_ensure_response(['success' => true, 'message' => 'Cursor deleted']);
            }
        }

        return new WP_Error('not_found', 'Cursor not found', ['status' => 404]);
    }

    public function wpcc_update_cursor_settings(WP_REST_Request $request) {
        $cursor_id = $request->get_param('id');
        $cursor_data = $request->get_json_params();

        if (!$cursor_id) {
            return new WP_Error('invalid_request', 'Missing cursor ID.', ['status' => 400]);
        }

        $cursor_settings = get_option('cursor_settings', []);
        $cursor_index = null;

        // Loop through each entry to find the cursor with matching ID
        foreach ($cursor_settings as $index => $entry) {
            if (isset($entry['cursor']['id']) && $entry['cursor']['id'] == $cursor_id) {
                $cursor_index = $index;
                break;
            }
        }

        if ($cursor_index === null) {
            return new WP_Error('not_found', 'Cursor not found.', ['status' => 404]);
        }

        // Merge existing data with new data to preserve other fields (like placement, timestamp)
        $cursor_settings[$cursor_index] = array_merge($cursor_settings[$cursor_index], $cursor_data);

        update_option('cursor_settings', $cursor_settings);

        return rest_ensure_response(['success' => true, 'data' => $cursor_settings[$cursor_index]]);
    }

    public function wpcc_reset_all_cursors() {
        // Delete the cursor_settings option
        delete_option('cursor_settings');

        // Delete the cursor_customizations option
        delete_option('cursor_customizations');

        return rest_ensure_response(['success' => true, 'message' => 'All cursor data has been reset.']);
    }
}