<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://hamidrezasepehr.com/
 * @since      1.0.0
 *
 * @package    Wpcc_Custom_Cursors
 * @subpackage Wpcc_Custom_Cursors/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Wpcc_Custom_Cursors
 * @subpackage Wpcc_Custom_Cursors/public
 * @author     Hamid Reza Sepehr <hamidsepehr4@gmail.com>
 */
class Wpcc_Custom_Cursors_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function wpcc_enqueue_styles() {
		$plugin_url = plugin_dir_url(__FILE__);

		// Define a mapping of cursor IDs to CSS files
		$cursor_styles = [
			10 => 'hephamage.css',
			13 => 'demetrobyte.css',
			14 => 'nyxamoke.css',
		];

		// Static variable to track enqueued styles
		static $enqueued_styles = [];

		// Check if preview mode is active
		if (isset($_GET['preview_cursor'])) {
			$cursor_config_json = sanitize_text_field($_GET['preview_cursor']);
			$cursor_config_json = wp_unslash($cursor_config_json);
			$cursor_config = json_decode($cursor_config_json, true);

			if ($cursor_config && isset($cursor_config['id'])) {
				$cursor_id = $cursor_config['id'];
				$style_file = isset($cursor_styles[$cursor_id]) ? $cursor_styles[$cursor_id] : 'wp-custom-cursors-public.css';
			} else {
				$style_file = 'wp-custom-cursors-public.css'; // Default fallback
			}

			// Enqueue the style only if it hasn't been enqueued yet
			if (!in_array($style_file, $enqueued_styles)) {
				wp_enqueue_style(
					$this->plugin_name . '-' . sanitize_title($style_file), // Unique handle
					$plugin_url . 'css/' . $style_file,
					[],
					$this->version,
					'all'
				);
				$enqueued_styles[] = $style_file;
			}


			return; // Stop execution to prevent live mode from loading
		}

		// **LIVE MODE (Non-preview mode)**
		$cursor_settings = get_option('cursor_settings', []);


		// Enqueue styles based on active cursor settings
		if (!empty($cursor_settings)) {
			foreach ($cursor_settings as $cursor) {
				$cursor_id = isset($cursor['cursor']['id']) ? intval($cursor['cursor']['id']) : null;

				if ($cursor_id && isset($cursor_styles[$cursor_id])) {
					$style_file = $cursor_styles[$cursor_id];

					// Enqueue the style only if it hasn't been enqueued yet
					if (!in_array($style_file, $enqueued_styles)) {
						wp_enqueue_style(
							$this->plugin_name . '-' . sanitize_title($style_file), // Unique handle
							$plugin_url . 'css/' . $style_file,
							[],
							$this->version,
							'all'
						);
						$enqueued_styles[] = $style_file;
					}
				}
				// Enqueue default style if no specific styles are needed or if it hasnt already been enqueued
				if (empty($cursor_styles[$cursor_id]) && !in_array("wp-custom-cursors-public.css", $enqueued_styles)) {
					wp_enqueue_style(
						$this->plugin_name . '-default',
						$plugin_url . 'css/wp-custom-cursors-public.css',
						[],
						$this->version,
						'all'
					);
					$enqueued_styles[] = "wp-custom-cursors-public.css";
				}
			}
		}
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function wpcc_enqueue_scripts() {
		// Check if preview mode is active
		$is_preview_mode = isset($_GET['preview_cursor']);
		
		if ($is_preview_mode) {
			// Sanitize and decode the cursor configuration
			$cursor_config_json = sanitize_text_field($_GET['preview_cursor']);
			$cursor_config_json = wp_unslash($cursor_config_json);
			$cursor_config = json_decode($cursor_config_json, true);
	
			if ($cursor_config && isset($cursor_config['id'])) {
				$cursor_scripts = [
					7 => 'dionytris.js',
					8 => 'aroneko.js',
					10 => 'hephamage.js',
					13 => 'demetrobyte.js',
					12 => 'hestellipse.js',
					14 => 'nyxamoke.js',
					15 => 'gaialoop.js',
					16 => 'aphorbit.js',
					17 => 'persephont.js',
					18 => 'iriadient.js',
				];
	
				$cursor_id = $cursor_config['id'];
				if (isset($cursor_scripts[$cursor_id])) {
					wp_enqueue_script(
						'wpcc-cursor-preview',
						plugins_url('./js/' . $cursor_scripts[$cursor_id], __FILE__),
						array(),
						'1.0.0',
						true
					);
				} else {
					wp_enqueue_script(
						'wpcc-cursor-preview',
						plugins_url('./js/cursor-preview.js', __FILE__),
						array(),
						'1.0.0',
						true
					);
				}
	
				// Pass the plugin's URL to the script
				wp_localize_script('wpcc-cursor-preview', 'wpccData', [
					'pluginUrl' => plugins_url('', __FILE__)
				]);
			}
	
			return; // Stop execution here to prevent live mode from loading
		}
	
		// Retrieve stored cursor settings
		$cursor_settings = get_option('cursor_settings', []);

		if (!empty($cursor_settings)) {
			// List of cursors with dedicated scripts
			$cursor_scripts = [
				7 => 'dionytris.js',
				8 => 'aroneko.js',
				10 => 'hephamage.js',
				13 => 'demetrobyte.js',
				12 => 'hestellipse.js',
				14 => 'nyxamoke.js',
				15 => 'gaialoop.js',
				16 => 'aphorbit.js',
				17 => 'persephont.js',
				18 => 'iriadient.js',
			];

			$enqueued_scripts = []; // Track enqueued scripts
			$default_script_needed = false; // Track if we need cursor-preview.js

			// Loop through all stored cursors
			foreach ($cursor_settings as $cursor) {
				$cursor_id = isset($cursor['cursor']['id']) ? intval($cursor['cursor']['id']) : null;

				if ($cursor_id) {
					if (isset($cursor_scripts[$cursor_id])) {
						// Enqueue cursor-specific JS if not already added
						$script_name = $cursor_scripts[$cursor_id];

						if (!in_array($script_name, $enqueued_scripts)) {
							wp_enqueue_script(
								'wpcc-cursor-' . $cursor_id,
								plugins_url('./js/' . $script_name, __FILE__),
								array(),
								'1.0.0',
								true
							);

							$enqueued_scripts[] = $script_name;
						}
					} else {
						// If cursor doesn't have a dedicated script, flag cursor-preview.js for inclusion
						$default_script_needed = true;
					}
				}
			}

			// Enqueue cursor-preview.js if needed
			if ($default_script_needed) {
				wp_enqueue_script(
					'wpcc-cursor-preview',
					plugins_url('./js/cursor-preview.js', __FILE__),
					array(),
					'1.0.0',
					true
				);
			}

			// Localize cursor settings for all scripts
			wp_localize_script('wpcc-cursor-preview', 'wpccData', [
				'pluginUrl' => plugins_url('', __FILE__),
				'settings' => $cursor_settings // Pass all cursor settings
			]);

			foreach ($enqueued_scripts as $script) {
				wp_localize_script('wpcc-cursor-' . array_search($script, $cursor_scripts), 'wpccData', [
					'pluginUrl' => plugins_url('', __FILE__),
					'settings' => $cursor_settings
				]);
			}
		}
	}

}
