function safeParseCursorConfig(config) {
    try {
        // Attempt to parse as raw JSON first
        return JSON.parse(config);
    } catch {
        try {
            // Attempt to decode and then parse
            return JSON.parse(decodeURIComponent(config));
        } catch (error) {
            console.error('Error decoding cursorConfig:', error);
            return null;
        }
    }
}
document.addEventListener('DOMContentLoaded', () => {
    const urlParams = new URLSearchParams(window.location.search);
    const cursorConfig = urlParams.get('preview_cursor');
    let cursors = [];

    class ImageCursor {
        constructor(config) {
            this.config = config;
            this.cursorElement = null;
            this.cursorImage = null; // Add this
            this.activePointX = (this.config.cursor.cursor.image.activePoint?.x || 0) / 100;
            this.activePointY = (this.config.cursor.cursor.image.activePoint?.y || 0) / 100;
            this.targetElement = null; // Add targetElement property
    
            this.init();
        }
    
        init() {
            // Function to set the cursor style based on showDefaultCursor
            this.setCursorStyle = (element, showDefaultCursor, cursorType = 'default') => {
                element.style.cursor = showDefaultCursor ? cursorType : 'none';
            };
    
            // Determine the target element based on config
            if (this.config.placement === 'specific_section' && this.config.section_identifier) {
                this.targetElement = document.querySelector(this.config.section_identifier);
                if (!this.targetElement) {
                    console.warn(`ImageCursor: Target element "${this.config.section_identifier}" not found.  Falling back to document.body.`);
                    this.targetElement = document.body;
                }
    
                // Apply showDefaultCursor to targetElement on specific_section
                const showDefaultCursor = String(this.config.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                this.setCursorStyle(this.targetElement, showDefaultCursor);
    
                // Add mouseenter and mouseleave listeners for specific sections
                this.targetElement.addEventListener('mouseenter', this.showCursor.bind(this));
                this.targetElement.addEventListener('mouseleave', this.hideCursor.bind(this));
    
            } else {
                this.targetElement = document.body; // Default to the whole body
    
                // Apply showDefaultCursor to body on entire_site
                const showDefaultCursor = String(this.config.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                this.setCursorStyle(this.targetElement, showDefaultCursor);
            }
    
            // Create the cursor wrapper and image element
            this.cursorElement = document.createElement('div');
            this.cursorImage = document.createElement('img');
    
            // Apply wrapper styles
            this.applyWrapperStyles(this.config.cursor.cursor.image);
    
            // Apply initial image styles
            this.applyImageStyles(this.config.cursor.cursor.image);
    
            // Append the image to the wrapper
            this.cursorElement.appendChild(this.cursorImage);
    
            // Add the wrapper to the target element (body or specific section)
            this.targetElement.appendChild(this.cursorElement);
    
            // Track mouse movement - use the target element for the event listener
            this.targetElement.addEventListener('mousemove', this.handleMouseMove.bind(this));
    
            // Handle hover states
            const hoverSelector = this.config.cursor.hover?.options?.hoverSelector || 'a'; // Default to 'a' if hoverSelector is not provided
            this.handleHover(hoverSelector);
    
            // Initially hide the cursor if it's a specific section
            if (this.config.placement === 'specific_section') {
                this.hideCursor();
            }
        }
    
        applyWrapperStyles(imageConfig) {
            const wrapperStyles = {
                position: 'fixed',
                top: '0',
                left: '0',
                pointerEvents: 'none',
                zIndex: '9999',
                opacity: imageConfig.opacity || 1,
                mixBlendMode: imageConfig.mixBlendMode || 'normal',
                transform: 'translate(-50%, -50%)',
                transition: `${imageConfig.transitionDuration || '0.2s'} ${imageConfig.transitionTimingFunction || 'ease-out'}`
            };
    
            Object.assign(this.cursorElement.style, wrapperStyles);
        }
    
    
        applyImageStyles(imageConfig) {
            const imageStyles = {
                width: imageConfig.width || '100px',
                backgroundColor: imageConfig.backgroundColor || 'transparent',
                borderRadius: imageConfig.borderRadius || '0',
                padding: imageConfig.padding || '0',
            };
            this.cursorImage.src = imageConfig.imageUrl;
            Object.assign(this.cursorImage.style, imageStyles);
        }
    
        handleMouseMove(e) {
            const offsetX = e.clientX - this.cursorElement.offsetWidth * this.activePointX;
            const offsetY = e.clientY - this.cursorElement.offsetHeight * this.activePointY;
    
            this.cursorElement.style.transform = `translate(${offsetX}px, ${offsetY}px)`;
        }
    
        handleHover(hoverSelector) {
            const hoverElements = document.querySelectorAll(hoverSelector);
            hoverElements.forEach((el) => {
                el.addEventListener('mouseenter', () => {
                    const showDefaultHoverCursor = String(this.config.cursor.hover?.options?.showDefaultCursor).toLowerCase() === 'true';
                    this.setCursorStyle(el, showDefaultHoverCursor, 'pointer');
    
                    // Apply styles from the hover configuration
                    this.applyImageStyles(this.config.cursor.hover.image);
                    this.applyWrapperStyles(this.config.cursor.hover.image); // Apply wrapper styles, including mixBlendMode!
                    this.updateActivePoint(this.config.cursor.hover.image.activePoint);
                    this.handleMouseMove({clientX: event.clientX, clientY: event.clientY});
    
                });
                el.addEventListener('mouseleave', () => {
                    const showDefaultCursor = String(this.config.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                    this.setCursorStyle(el, showDefaultCursor);
    
                    // Revert to the original cursor styles
                    this.applyImageStyles(this.config.cursor.cursor.image);
                    this.applyWrapperStyles(this.config.cursor.cursor.image); // Revert wrapper styles
                    this.updateActivePoint(this.config.cursor.cursor.image.activePoint);
                    this.handleMouseMove({clientX: event.clientX, clientY: event.clientY});
                });
            });
        }
    
        updateActivePoint(activePoint) {
            if (!activePoint) return;
            this.activePointX = (activePoint.x || 0) / 100; // Convert to ratio
            this.activePointY = (activePoint.y || 0) / 100; // Convert to ratio
        }
    
        showCursor() {
            this.cursorElement.style.visibility = 'visible';
        }
    
        hideCursor() {
            this.cursorElement.style.visibility = 'hidden';
        }
    }

    if (cursorConfig) {
        const parsedConfig = safeParseCursorConfig(cursorConfig);
        // Ensure it's an array, even if it's a single object
        cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
        
        // Add default placement and section_identifier for preview mode
        cursors = cursors.map(cursor => ({
            cursor: {
                id: cursor.id || null, 
                name: cursor.name || 'Default Cursor',
                type: cursor.type || 'html', 
                hoverSelector: cursor.hoverSelector || 'a', 
                cursor: cursor.cursor || {}, 
                hover: cursor.hover || {} 
            },
            placement: 'entire_site', 
            section_identifier: '', 
            timestamp: Date.now() 
        }));
    } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
        cursors = wpccData.settings;
    }

    if (Array.isArray(cursors)) {
        const aphorbitConfigs = []; // Array to store Aphorbit configurations
        cursors.forEach((cursor) => {
            if (cursor?.cursor?.name == "Aphorbit") {
                aphorbitConfigs.push(cursor);
            }
        });
        if (aphorbitConfigs.length > 0) {
            aphorbitConfigs.forEach(config => {
                new ImageCursor(config); // Create a new instance for each config
            });
        } else {
            console.log("No Aphorbit cursors found in the configuration.");
        }
    }
});