function safeParseCursorConfig(config) {
    try {
        // Attempt to parse as raw JSON first
        return JSON.parse(config);
    } catch {
        try {
            // Attempt to decode and then parse
            return JSON.parse(decodeURIComponent(config));
        } catch (error) {
            console.error('Error decoding cursorConfig:', error);
            return null;
        }
    }
}

document.addEventListener('DOMContentLoaded', () => {
    const urlParams = new URLSearchParams(window.location.search);
    const cursorConfig = urlParams.get('preview_cursor');
    let cursors = [];

    if (cursorConfig) {
        const parsedConfig = safeParseCursorConfig(cursorConfig);
        // Ensure it's an array, even if it's a single object
        cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
        
        // Add default placement and section_identifier for preview mode
        cursors = cursors.map(cursor => ({
            cursor: {
                id: cursor.id || null, 
                name: cursor.name || 'Default Cursor',
                type: cursor.type || 'html', 
                hoverSelector: cursor.hoverSelector || 'a', 
                cursor: cursor.cursor || {}, 
                hover: cursor.hover || {} 
            },
            placement: 'entire_site', 
            section_identifier: '', 
            timestamp: Date.now() 
        }));
    } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
        cursors = wpccData.settings;
    }

    if (Array.isArray(cursors)) {
        class DynamicCursor {
            constructor(cursors) {
                this.allCursors = []; // Store all SingleCursor instances
                this.mouse = { x: 0, y: 0 }; // Mouse position
    
                // Initialize all cursors
                cursors.forEach(cursor => {
                    this.allCursors.push(new SingleCursor(cursor, this.mouse, this));
                });
    
                // Track mouse movement
                document.addEventListener('mousemove', this.handleMouseMove.bind(this));
    
                this.init();
            }
    
            init() {
    
            }
    
            shouldActivateCursor(cursor) {
                // Check the placement
                if (cursor.placement === 'entire_site') {
                    return true;
                } else if (cursor.placement === 'specific_section' && document.querySelector(cursor.section_identifier)) {
                    return true;
                }
                return false; // If no valid placement, cursor won't be activated
            }
    
            handleMouseMove(event) {
                this.mouse.x = event.clientX;
                this.mouse.y = event.clientY;
                // Update positions of all cursors
                this.allCursors.forEach(cursor => cursor.updatePosition(this.mouse));
            }
        }
        class SingleCursor {
            constructor(cursor, mouse, dynamicCursor) {
                this.cursor = cursor; // The individual cursor object
                this.mouse = mouse; // Shared mouse position
                this.dynamicCursor = dynamicCursor; // Reference to parent manager
                this.wrapper = null; // Wrapper for cursor
                this.elements = {};
                this.isHovering = false;
                this.hoveredElement = null; // Store the hovered element
                this.sectionElements = []; // Store associated section elements
                this.parentCursorsHidden = []; // Track hidden parent cursors
                this.init();
            }
    
            init() {
                this.createWrapper();
    
                if (this.cursor.cursor.cursor.outer) {
                    this.elements.outer = this.createHTMLElement(this.cursor.cursor.cursor.outer, 'outer');
                }
                if (this.cursor.cursor.cursor.inner) {
                    this.elements.inner = this.createHTMLElement(this.cursor.cursor.cursor.inner, 'inner');
                }
    
                // Add hover event listeners if applicable
                if (this.cursor.cursor.hover?.options?.hoverSelector) {
                    const hoverElements = document.querySelectorAll(this.cursor.cursor.hover?.options?.hoverSelector);
                    hoverElements.forEach(el => {
                        el.addEventListener('mouseenter', this.handleMouseEnter.bind(this, el));
                        el.addEventListener('mouseleave', this.handleMouseLeave.bind(this, el));
                    });
                }
    
                // Add event listeners to the section identified by section_identifier
                if (this.cursor.placement === 'specific_section' && this.cursor.section_identifier) {
                    this.sectionElements = document.querySelectorAll(this.cursor.section_identifier);
                    this.sectionElements.forEach(sectionElement => {
                        sectionElement.addEventListener('mouseenter', this.handleSectionMouseEnter.bind(this));
                        sectionElement.addEventListener('mouseleave', this.handleSectionMouseLeave.bind(this));
                    });
                }
    
                // If placement is entire site, hide the cursor on the entire body
                if (this.cursor.placement === 'entire_site') {
                    this.sectionElements = [document.body];
                }
    
                // If placement is specific_section, hide the cursor initially
                if (this.cursor.placement === 'specific_section') {
                    this.hide();
                }
    
                // Initialize section elements cursor visibility
                this.updateSectionElementsCursorVisibility(String(this.cursor.cursor.cursor.options?.showDefaultCursor).toLowerCase() === 'true');
            }
    
            createWrapper() {
                if (!this.wrapper) {
                    this.wrapper = document.createElement('div');
                    this.wrapper.className = `cursor-wrapper ${this.cursor.cursor.name.toLowerCase()}`;
                    this.wrapper.style.position = 'relative';
                    document.body.appendChild(this.wrapper);
                }
            }
    
            updateSectionElementsCursorVisibility(showDefaultCursor) {
                const cursorStyle = showDefaultCursor ? 'default' : 'none';
                this.sectionElements.forEach(element => {
                    if (element) {
                        element.style.cursor = cursorStyle;
                    }
                });
            }
    
            createHTMLElement(styles, className) {
                const element = document.createElement('div');
                element.className = `custom-cursor ${className}`;
    
                // Get a list of valid CSS properties
                const cssPropertyList = new Set(Object.keys(document.createElement('div').style));
    
                // Separate CSS styles and non-CSS styles
                const cssStyles = {};
                const cssVariables = {};
    
                for (const [key, value] of Object.entries(styles)) {
                    if (cssPropertyList.has(key)) {
                        // It's a valid CSS property
                        cssStyles[key] = value;
                    } else {
                        // Treat as a custom property, convert to CSS variable
                        if (key === 'text') {
                            // Special handling for `content`-related variables
                            cssVariables[`--${key}`] = `"${value}"`; // Add quotes for CSS
    
                            // Create a div for the text content
                            const textDiv = document.createElement('div');
                            textDiv.textContent = value;
                            textDiv.className = 'cursor-text'; // Optional: Add a class for styling
                            element.appendChild(textDiv);
                        } else {
                            cssVariables[`--${key}`] = value;
                        }
                    }
                }
    
                // Assign default top and left values
                const defaultTop = cssStyles.top || '0px';
                const defaultLeft = cssStyles.left || '0px';
    
                Object.assign(element.style, {
                    position: 'fixed',
                    boxSizing: 'border-box',
                    top: cssStyles.height === 'auto' ? defaultTop : `-${parseFloat(cssStyles.height) / 2}px`,
                    left: cssStyles.width === 'auto' ? defaultLeft : `-${parseFloat(cssStyles.width) / 2}px`,
                    pointerEvents: 'none',
                    ...styles
                });
    
                // Assign CSS variables
                for (const [key, value] of Object.entries(cssVariables)) {
                    element.style.setProperty(key, value);
                }
    
                this.wrapper.appendChild(element);
                return element;
            }
    
            updatePosition(mouse) {
                const outerSnap = String(this.cursor.cursor.hover?.outer?.snap).toLowerCase() === 'true';
                const shouldMoveOuter = !(this.isHovering && outerSnap);
    
                if (this.elements.outer) {
                    if (shouldMoveOuter) {
                        this.elements.outer.style.transform = `translate(${mouse.x}px, ${mouse.y}px)`;
                    }
                }
                if (this.elements.inner) {
                    this.elements.inner.style.transform = `translate(${mouse.x}px, ${mouse.y}px)`;
                }
            }
    
            handleSectionMouseEnter() {
                this.show(); // Show cursor on section hover
    
                // Hide parent cursors whose sections contain this cursor's sections
                this.parentCursorsHidden = [];
                this.dynamicCursor.allCursors.forEach(otherCursor => {
                    if (otherCursor !== this && otherCursor.sectionElements.length > 0) {
                        const isParent = Array.from(this.sectionElements).some(childEl =>
                            Array.from(otherCursor.sectionElements).some(parentEl =>
                                parentEl.contains(childEl)
                            )
                        );
                        if (isParent) {
                            otherCursor.hide();
                            this.parentCursorsHidden.push(otherCursor);
                        }
                    }
                });
            }
    
            handleSectionMouseLeave() {
                this.hide(); // Hide cursor when leaving section
    
                // Show previously hidden parent cursors
                this.parentCursorsHidden.forEach(parentCursor => parentCursor.show());
                this.parentCursorsHidden = [];
            }
    
            handleMouseEnter(element) {
                this.isHovering = true;
                this.hoveredElement = element; // Store the hovered element
                this.updateStyles();
                this.handleSnap(element); // Snap on mouse enter
    
                const showDefaultHoverCursor = String(this.cursor.cursor.hover?.options?.showDefaultCursor).toLowerCase() === 'true';
                element.style.cursor = showDefaultHoverCursor ? 'pointer' : 'none';
            }
    
            handleMouseLeave(element) {
                this.isHovering = false;
                this.updateStyles();
                this.resetSnap(); // Reset snapped elements
                const showDefaultCursor = String(this.cursor.cursor.options?.showDefaultCursor).toLowerCase() === 'true';
                element.style.cursor = showDefaultCursor ? 'default' : 'none';
            }
    
            handleSnap(targetElement) {
                const rect = targetElement.getBoundingClientRect();
                const centerX = rect.left + rect.width / 2;
                const centerY = rect.top + rect.height / 2;
                const outerSnap = String(this.cursor.cursor.hover?.outer?.snap).toLowerCase() === 'true';
    
                // Handle outer cursor snap
                if (this.elements.outer && outerSnap) {
                    const padding = this.cursor.cursor.hover?.outer?.padding || 0;
                    const adjustedWidth = rect.width + padding * 2 + 10; //10px extra padding as default
                    const adjustedHeight = rect.height + padding * 2 + 10; //10px extra padding as default
    
                    this.elements.outer.style.transform = `translate(${centerX}px, ${centerY}px)`;
                    this.elements.outer.style.width = `${adjustedWidth}px`;
                    this.elements.outer.style.height = `${adjustedHeight}px`;
    
                    // Adjust left and top to account for the new dimensions
                    this.elements.outer.style.left = `-${adjustedWidth / 2}px`;
                    this.elements.outer.style.top = `-${adjustedHeight / 2}px`;
                }
            }
    
            resetSnap() {
                const outerSnap = String(this.cursor.cursor.hover?.outer?.snap).toLowerCase() === 'true';
    
                if (this.elements.outer && outerSnap) {
                    this.elements.outer.style.width = this.cursor.cursor.cursor.outer.width;
                    this.elements.outer.style.height = this.cursor.cursor.cursor.outer.height;
                    this.elements.outer.style.left = `-${parseFloat(this.cursor.cursor.cursor.outer.width) / 2}px`;
                    this.elements.outer.style.top = `-${parseFloat(this.cursor.cursor.cursor.outer.height) / 2}px`;
                    this.elements.outer.style.transform = `translate(${this.mouse.x}px, ${this.mouse.y}px)`;
                }
            }
    
            show() {
                if (this.elements.outer) {
                    this.elements.outer.style.display = 'block';
                }
                if (this.elements.inner) {
                    this.elements.inner.style.display = 'block';
                }
            }
    
            hide() {
                if (this.elements.outer) {
                    this.elements.outer.style.display = 'none';
                }
                if (this.elements.inner) {
                    this.elements.inner.style.display = 'none';
                }
            }
    
            updateStyles() {
                const applyStyles = (element, baseStyles, hoverStyles, isHovering) => {
                    let styles = { ...baseStyles };  // Start with base styles
            
                    if (isHovering && hoverStyles) {
                        styles = { ...styles, ...hoverStyles }; // Merge hover styles on top
                    }
            
                    if (styles) {
                        const cssPropertyList = new Set(Object.keys(document.createElement('div').style));
                        const cssStyles = {};
                        const cssVariables = {};
            
                        for (const key in styles) {
                            if (styles.hasOwnProperty(key)) {
                                const value = styles[key];
            
                                if (cssPropertyList.has(key)) {
                                    // It's a valid CSS property
                                    cssStyles[key] = value;
                                } else {
                                    // Treat as a custom property, convert to CSS variable
                                    if (key === 'text') {
                                        cssVariables[`--${key}`] = `"${value}"`; // Add quotes for CSS
            
                                        // Update text content of the child element
                                        const textDiv = element.querySelector('.cursor-text');
                                        if (textDiv) {
                                            textDiv.textContent = value;
                                        } else {
                                            // Create text div if it doesn't exist
                                            const newTextDiv = document.createElement('div');
                                            newTextDiv.textContent = value;
                                            newTextDiv.className = 'cursor-text';
                                            element.appendChild(newTextDiv);
                                        }
                                    } else {
                                        cssVariables[`--${key}`] = value;
                                    }
                                }
                            }
                        }
            
                        // Apply CSS styles
                        Object.assign(element.style, cssStyles);
            
                        // Apply CSS variables
                        for (const [key, value] of Object.entries(cssVariables)) {
                            element.style.setProperty(key, value);
                        }
            
                        // Only update position when isHovering is false or outerSnap is false
                        const outerSnap = String(this.cursor.cursor.hover?.outer?.snap).toLowerCase() === 'true';
                        if (!isHovering || !outerSnap) {
                            Object.assign(element.style, {
                                top: `-${parseFloat(styles.height) / 2}px`,
                                left: `-${parseFloat(styles.width) / 2}px`,
                            });
                        }
                    }
                };
            
                if (this.elements.outer) {
                    applyStyles(
                        this.elements.outer,
                        this.cursor.cursor.cursor.outer,
                        this.cursor.cursor.hover?.outer,
                        this.isHovering
                    );
                }
            
                if (this.elements.inner) {
                    applyStyles(
                        this.elements.inner,
                        this.cursor.cursor.cursor.inner,
                        this.cursor.cursor.hover?.inner,
                        this.isHovering
                    );
                }
            }
        }
        const $cursor_names = {
            7: 'dionytris',
            8: 'aroneko',
            10: 'hephamage',
            13: 'demetrobyte',
            12: 'hestellipse',
            14: 'nyxamoke',
            15: 'gaialoop',
            16: 'aphorbit',
            17: 'persephont',
            18: 'iriadient',
        };
    
        const filteredCursors = [];
    
        cursors.forEach(cursor => {
            // Check if the cursor's name is NOT in the $cursor_names array
            const cursorId = cursor?.cursor?.id;
            if (!cursorId || !$cursor_names[cursorId]) {
                filteredCursors.push(cursor);
            }
        });
    
        // Only create DynamicCursor if there are filtered cursors
        if (filteredCursors.length > 0) {
            console.log(JSON.stringify(filteredCursors));
            const dynamicCursor = new DynamicCursor(filteredCursors);
        } else {
            console.log("No cursors found that are not in $cursor_names.");
        }
    
    } else {
        console.warn("No valid cursor settings found.");
    }
});