function safeParseCursorConfig(config) {
    try {
        // Attempt to parse as raw JSON first
        return JSON.parse(config);
    } catch {
        try {
            // Attempt to decode and then parse
            return JSON.parse(decodeURIComponent(config));
        } catch (error) {
            console.error('Error decoding cursorConfig:', error);
            return null;
        }
    }
}

document.addEventListener('DOMContentLoaded', () => {
    const urlParams = new URLSearchParams(window.location.search);
    const cursorConfig = urlParams.get('preview_cursor');
    let cursors = [];

    function initializeGaialoopCursors(cursorConfigs) {
        cursorConfigs.forEach(cursorConfig => {
            const SVGNS = 'http://www.w3.org/2000/svg';
            const EASE = cursorConfig.cursor.cursor.config?.ease ?? 0.7;
    
            const pointer = {
                x: window.innerWidth / 2,
                y: window.innerHeight / 2
            };
    
            let isHovering = false;
            let svgContainers = [];
    
            // Function to set the cursor style based on showDefaultCursor
            const setCursorStyle = (element, showDefaultCursor, cursorType = 'default') => {
                element.style.cursor = showDefaultCursor ? cursorType : 'none';
            };
    
            function updatePointer(event) {
                pointer.x = event.clientX;
                pointer.y = event.clientY;
            }
    
            function createLine(leader) {
                const line = document.createElementNS(SVGNS, 'line');
                let currentX = pointer.x;
                let currentY = pointer.y;
                let currentWidth = cursorConfig.cursor.cursor.config?.width ?? 60;
                const originalWidth = currentWidth;
    
                line.setAttribute('x1', '0');
                line.setAttribute('y1', '0');
                line.setAttribute('x2', '0');
                line.setAttribute('y2', '0');
    
                function updatePosition() {
                    const leaderX = typeof leader === 'function' ? leader('x') : pointer.x;
                    const leaderY = typeof leader === 'function' ? leader('y') : pointer.y;
    
                    // Update position with EASE
                    currentX += (leaderX - currentX) * EASE;
                    currentY += (leaderY - currentY) * EASE;
    
                    // Update width with a slower transition
                    const hoverWidth = cursorConfig.cursor.hover.config?.width ?? 20;
                    const targetWidth = isHovering ? Number(hoverWidth) : originalWidth;
                    const widthEase = 0.08; // Slower easing factor for width
                    currentWidth += (targetWidth - currentWidth) * widthEase;
    
                    // Apply the transformations
                    line.setAttribute('transform', `translate(${currentX},${currentY})`);
                    line.setAttribute('x2', leaderX - currentX);
                    line.setAttribute('y2', leaderY - currentY);
    
                    // Apply width with the calculated value
                    line.style.strokeWidth = currentWidth;
    
                    // Hide trail on hover, keep the first circle
                    line.style.opacity = isHovering ? (leader === pointer ? '1' : '1') : '1';
    
                    requestAnimationFrame(updatePosition);
                }
    
                updatePosition();
    
                return {
                    element: line,
                    getProperty: (prop) => prop === 'x' ? currentX : currentY
                };
            }
    
            function createTrail(color, length = 10) {
                const svg = document.createElementNS(SVGNS, 'svg');
                svg.style.position = 'fixed';
                svg.style.top = '0';
                svg.style.left = '0';
                svg.style.width = '100%';
                svg.style.height = '100%';
                svg.style.pointerEvents = 'none';
                svg.style.zIndex = '1000';
                svg.style.visibility = 'hidden'; // Initially hidden by default
    
                // Create outer circle for hover state
                const outerCircle = document.createElementNS(SVGNS, 'circle');
                const radius = cursorConfig.cursor.hover.config?.strokeRadius ?? 30;
                const circumference = 2 * Math.PI * radius;
    
                outerCircle.setAttribute('r', radius.toString());
                outerCircle.setAttribute('cx', pointer.x.toString());
                outerCircle.setAttribute('cy', pointer.y.toString());
                outerCircle.style.fill = 'none';
                outerCircle.style.stroke = cursorConfig.cursor.hover.config?.strokeColor ?? 'red';
                outerCircle.style.strokeWidth = cursorConfig.cursor.hover.config?.strokeWidth ?? '2';
                outerCircle.style.strokeDasharray = circumference.toString(); // Set stroke-dasharray to the circle's circumference
                outerCircle.style.strokeDashoffset = circumference.toString(); // Initially hide the stroke
                outerCircle.style.transition = 'stroke-dashoffset 0.6s ease-out, opacity 0.6s ease-out';
                outerCircle.style.opacity = '0'; // Start hidden
                svg.appendChild(outerCircle);
    
                let leader = (prop) => prop === 'x' ? pointer.x : pointer.y;
                const lines = [];
    
                for (let i = 0; i < length; i++) {
                    const lineObj = createLine(leader);
                    const line = lineObj.element;
    
                    line.style.stroke = color;
                    line.style.strokeLinecap = 'round';
                    svg.appendChild(line);
    
                    lines.push(lineObj);
                    leader = lineObj.getProperty;
                }
    
                document.body.appendChild(svg);
                svgContainers.push({ svg, outerCircle });
    
                // Update function for hover effect
                function updateOuterCircle() {
                    outerCircle.setAttribute('cx', pointer.x);
                    outerCircle.setAttribute('cy', pointer.y);
    
                    if (isHovering) {
                        outerCircle.style.strokeDashoffset = '0'; // Draw the circle
                        outerCircle.style.opacity = '1'; // Make it visible
                    } else {
                        outerCircle.style.strokeDashoffset = circumference.toString(); // Hide the stroke
                        outerCircle.style.opacity = '0'; // Make it invisible
                    }
    
                    requestAnimationFrame(updateOuterCircle);
                }
                updateOuterCircle();
    
                return svg; // Return the created SVG element.
            }
    
            // Hover effects (moved inside to make them scoped)
            function handleLinkHover() {
                isHovering = true;
            }
    
            function handleLinkLeave() {
                isHovering = false;
            }
    
            // **Crucially: Define addHoverListeners here**
            function addHoverListeners(element, hoverSelector) {
                element.querySelectorAll(hoverSelector).forEach(link => {
                    link.addEventListener('mouseenter', () => { // Arrow function to preserve 'this' context
                        handleLinkHover();
                        const showDefaultHoverCursor = String(cursorConfig.cursor.hover?.options?.showDefaultCursor).toLowerCase() === 'true';
                        setCursorStyle(link, showDefaultHoverCursor, 'pointer');
                    });
                    link.addEventListener('mouseleave', () => { // Arrow function to preserve 'this' context
                        handleLinkLeave();
                        const showDefaultCursor = String(cursorConfig.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                        setCursorStyle(link, showDefaultCursor);
                    });
                });
            }
    
            window.addEventListener('pointerdown', updatePointer);
            window.addEventListener('pointermove', updatePointer);
    
            const color1 = cursorConfig.cursor.cursor.shape1?.color ?? 'blue';
            const color2 = cursorConfig.cursor.cursor.shape2?.color ?? 'yellow';
            const color3 = cursorConfig.cursor.cursor.shape3?.color ?? 'red';
    
            const length1 = cursorConfig.cursor.cursor.shape1?.length ?? 30;
            const length2 = cursorConfig.cursor.cursor.shape2?.length ?? 20;
            const length3 = cursorConfig.cursor.cursor.shape3?.length ?? 10;
    
            // Create the trails and store the SVG elements.
            const svg1 = createTrail(color1, length1);
            const svg2 = createTrail(color2, length2);
            const svg3 = createTrail(color3, length3);
    
            // Determine where to apply the cursor effect
            if (cursorConfig.placement === 'entire_site') {
                // For entire site, the SVGs should be visible
                svg1.style.visibility = 'visible';
                svg2.style.visibility = 'visible';
                svg3.style.visibility = 'visible';
    
                const showDefaultCursor = String(cursorConfig.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                setCursorStyle(document.body, showDefaultCursor);
                const hoverSelector = cursorConfig.cursor.hover?.options?.hoverSelector;
    
                addHoverListeners(document.body, hoverSelector);
    
            } else if (cursorConfig.placement === 'specific_section') {
                const targetElements = document.querySelectorAll(cursorConfig.section_identifier);
    
                targetElements.forEach(targetElement => {
    
                    // Add event listeners to the target element.
                    targetElement.addEventListener('mouseenter', () => {
                        // Make SVGs visible when mouse enters the target element
                        svg1.style.visibility = 'visible';
                        svg2.style.visibility = 'visible';
                        svg3.style.visibility = 'visible';
                    });
    
                    targetElement.addEventListener('mouseleave', () => {
                        // Hide SVGs when mouse leaves the target element
                        svg1.style.visibility = 'hidden';
                        svg2.style.visibility = 'hidden';
                        svg3.style.visibility = 'hidden';
                    });
    
                    const showDefaultCursor = String(cursorConfig.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                    setCursorStyle(targetElement, showDefaultCursor);
    
                    const hoverSelector = cursorConfig.cursor.hover?.options?.hoverSelector;
                    addHoverListeners(targetElement, hoverSelector);
                });
            }
        });
    }

    if (cursorConfig) {
        const parsedConfig = safeParseCursorConfig(cursorConfig);
        // Ensure it's an array, even if it's a single object
        cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
        
        // Add default placement and section_identifier for preview mode
        cursors = cursors.map(cursor => ({
            cursor: {
                id: cursor.id || null, 
                name: cursor.name || 'Default Cursor',
                type: cursor.type || 'html', 
                hoverSelector: cursor.hoverSelector || 'a', 
                cursor: cursor.cursor || {}, 
                hover: cursor.hover || {} 
            },
            placement: 'entire_site', 
            section_identifier: '', 
            timestamp: Date.now() 
        }));
    } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
        cursors = wpccData.settings;
    }

    if (Array.isArray(cursors)) {
        const gaialoopConfigs = []; // Array to store Gaialoop configurations
        cursors.forEach((cursor) => {
            if (cursor?.cursor?.name == "Gaialoop") {
                gaialoopConfigs.push(cursor);
            }
        });
        if (gaialoopConfigs.length > 0) {
             // Pass the Gaialoop configurations 
             initializeGaialoopCursors(gaialoopConfigs);
        } else {
            console.log("No Gaialoop cursors found in the configuration.");
        }
    }
});