function safeParseCursorConfig(config) {
    try {
        // Attempt to parse as raw JSON first
        return JSON.parse(config);
    } catch {
        try {
            // Attempt to decode and then parse
            return JSON.parse(decodeURIComponent(config));
        } catch (error) {
            console.error('Error decoding cursorConfig:', error);
            return null;
        }
    }
}

document.addEventListener('DOMContentLoaded', function () {
    const urlParams = new URLSearchParams(window.location.search);
    const cursorConfig = urlParams.get('preview_cursor');
    let cursors = [];

    function imageTrailEffect(cursorConfigs) {

        // Helper functions
        const MathUtils = {
            lerp: (a, b, n) => (1 - n) * a + n * b,
            distance: (x1, y1, x2, y2) => Math.hypot(x2 - x1, y2 - y1),
            getRandomFloat: (min, max) => Math.random() * (max - min) + min,
        }
    
        let winsize;
        const calcWinsize = () => {
            winsize = {
                width: window.innerWidth,
                height: window.innerHeight
            };
        };
        calcWinsize();
        window.addEventListener('resize', calcWinsize);
    
        const getMousePos = (ev) => {
            let posx = 0;
            let posy = 0;
            if (!ev) ev = window.event;
            if (ev.clientX || ev.clientY) {
                posx = ev.clientX;
                posy = ev.clientY;
            }
            return {
                x: posx,
                y: posy
            };
        }
    
        let mousePos, lastMousePos, cacheMousePos;
        mousePos = lastMousePos = cacheMousePos = {
            x: 0,
            y: 0
        };
    
        // Define mousemove listener here, it will be reassigned based on scope
        let mouseMoveListener = ev => mousePos = getMousePos(ev);
    
        const getMouseDistance = () => MathUtils.distance(mousePos.x, mousePos.y, lastMousePos.x, lastMousePos.y);
    
        class Image {
            constructor(el) {
                this.DOM = {
                    el: el
                };
                this.defaultStyle = {
                    scale: 1,
                    x: 0,
                    y: 0,
                    opacity: 0
                };
                this.initEvents();
    
                // Ensure dimensions are ready
                if (this.DOM.el.complete) {
                    // If the image is already loaded
                    this.getRect();
                } else {
                    // Wait for the image to load
                    this.DOM.el.addEventListener('load', () => this.getRect());
                }
            }
    
            initEvents() {
                window.addEventListener('resize', () => this.resize());
            }
    
            resize() {
                Object.assign(this.DOM.el.style, {
                    transform: `translate(0,0) scale(1)`,
                    opacity: 0
                });
                this.getRect();
            }
    
            getRect() {
                this.rect = this.DOM.el.getBoundingClientRect();
            }
    
            isActive() {
                return this.DOM.el.style.opacity !== '0';
            }
        }
    
        class ImageTrail {
            constructor(container, cursor) {
                this.container = container;
                this.cursor = cursor;
                this.DOM = {
                    content: this.createContent(cursor)
                };
                this.images = [];
                [...this.DOM.content.querySelectorAll('img')].forEach(img => this.images.push(new Image(img)));
                this.imagesTotal = this.images.length;
                this.imgPosition = 0;
                this.zIndexVal = 1;
                this.threshold = cursor?.cursor?.cursor?.config?.threshold ?? 100;
                this.render();
    
                // Add mousemove listener to the container
                this.container.addEventListener('mousemove', mouseMoveListener);
            }
    
            createContent(cursor) {
                // Create content container
                const content = document.createElement('div');
                content.className = 'content';
                this.container.appendChild(content);
    
                // Dynamically construct image URLs using the plugin URL
                const pluginUrl = wpccData.pluginUrl; // This is passed from PHP
                const defaultImageSources = [
                    `${pluginUrl}/img/1.png`,
                    `${pluginUrl}/img/2.png`,
                    `${pluginUrl}/img/3.png`,
                    `${pluginUrl}/img/4.png`,
                    `${pluginUrl}/img/5.png`,
                    `${pluginUrl}/img/6.png`,
                    `${pluginUrl}/img/7.png`,
                    `${pluginUrl}/img/8.png`,
                    `${pluginUrl}/img/9.png`,
                    `${pluginUrl}/img/10.png`,
                    `${pluginUrl}/img/11.png`,
                    `${pluginUrl}/img/12.png`,
                    `${pluginUrl}/img/13.png`,
                    `${pluginUrl}/img/14.png`,
                    `${pluginUrl}/img/15.png`
                ];
    
                // Set CSS variables for content wrapper
                const {
                    width,
                    filter,
                    blendingMode,
                    zIndex
                } = cursor?.cursor?.cursor?.config || {};
                if (width) content.style.setProperty('--width', width);
                if (filter) content.style.setProperty('--filter', filter);
                if (blendingMode) content.style.setProperty('--blending-mode', blendingMode);
                if (zIndex) {
                    content.style.position = 'relative';
                    content.style.setProperty('--z-index', zIndex);
                }
                // Use cursor imageUrls if available, otherwise use default sources
                const imageUrls = cursor?.cursor?.cursor?.config?.imageUrls || defaultImageSources.join(',');

                
    
                const imageSources = imageUrls.split(',').map(url => url.trim()); // Split into array and trim
    
                imageSources.forEach(src => {
                    const img = document.createElement('img');
                    img.className = 'content__img';
                    img.src = src; // Use the source from the array
                    img.alt = 'Some image';
                    content.appendChild(img); // Append the image to the container
                });
                return content;
            };
    
            render() {
                let distance = getMouseDistance();
    
                cacheMousePos.x = MathUtils.lerp(cacheMousePos.x || mousePos.x, mousePos.x, 0.1);
                cacheMousePos.y = MathUtils.lerp(cacheMousePos.y || mousePos.y, mousePos.y, 0.1);
    
                if (distance > this.threshold) {
                    this.showNextImage();
                    ++this.zIndexVal;
                    this.imgPosition = this.imgPosition < this.imagesTotal - 1 ? this.imgPosition + 1 : 0;
                    lastMousePos = mousePos;
                }
    
                let isIdle = true;
                for (let img of this.images) {
                    if (img.isActive()) {
                        isIdle = false;
                        break;
                    }
                }
    
                if (isIdle && this.zIndexVal !== 1) {
                    this.zIndexVal = 1;
                }
    
                requestAnimationFrame(() => this.render());
            }
    
            showNextImage() {
                const img = this.images[this.imgPosition];
                const el = img.DOM.el;
    
                // Calculate the initial position
                const initialX = mousePos.x - img.rect.width / 2;
                const initialY = mousePos.y - img.rect.height / 2;
    
    
                // Ensure the image starts fresh
                el.style.transition = 'none'; // Disable transition temporarily
                el.style.opacity = 0; // Reset opacity
                el.style.transform = `translate(${initialX}px, ${initialY}px)`; // Start scaled down at cursor
    
                // Force reflow to apply the initial state
                el.offsetHeight;
    
                // Fade in and grow smoothly to scale(1)
                el.style.transition = 'transform 0.9s cubic-bezier(0.19, 1, 0.22, 1)';
                el.style.opacity = 1;
                el.style.transform = `translate(${initialX}px, ${initialY}px) scale(1)`;
                el.style.zIndex = this.zIndexVal;
    
                // Update the position smoothly as the cursor moves
                requestAnimationFrame(() => {
                    el.style.transform = `translate(${mousePos.x - img.rect.width / 2}px, ${mousePos.y - img.rect.height / 2}px) scale(1)`;
                });
    
                // Define timeout behaviors based on cursor style
                const timeoutBehaviors = {
                    '1': () => {
                        setTimeout(() => {
                            el.style.transition = 'opacity 1s ease-out, transform 1s cubic-bezier(0.23, 1, 0.32, 1)';
                            el.style.opacity = 0;
                            el.style.transform = `translate(${initialX}px, ${initialY}px) scale(0.2)`;
                        }, 400);
                    },
                    '2': () => {
                        setTimeout(() => {
                            el.style.transition = 'opacity 0.8s ease-out, transform 0.8s cubic-bezier(0.86, 0, 0.07, 1)';
                            el.style.opacity = 0;
                            el.style.transformOrigin = 'center';
                            el.style.transform = `translate(${initialX}px, ${initialY}px) scale(2)`;
                        }, 800);
                    },
                    '3': () => {
                        // Translate the image down off-screen after 0.4s
                        setTimeout(() => {
                            el.style.transition = 'transform 800ms ease-in-out';
                            el.style.transform = `translate(${initialX}px, ${
                                mousePos.y - img.rect.height / 2 + window.innerHeight + img.rect.height / 2
                            }px)`;
                        }, 400);
                    },
                    '4': () => {
                        // Translate the image off-screen with random rotation after 0.6s
                        setTimeout(() => {
                            const randomX = MathUtils.getRandomFloat(
                                -1 * (winsize.width + img.rect.width / 2),
                                winsize.width + img.rect.width / 2
                            );
                            const randomY = MathUtils.getRandomFloat(
                                -1 * (winsize.height + img.rect.height / 2),
                                winsize.height + img.rect.height / 2
                            );
                            const randomRotation = MathUtils.getRandomFloat(-40, 40);
                
                            el.style.transition = 'opacity 1s ease-out, transform 1s ease-out';
                            el.style.opacity = 0;
                            el.style.transform = `
                                translate(${mousePos.x + randomX}px, ${mousePos.y + randomY}px) 
                                rotate(${randomRotation}deg)
                            `;
                        }, 400);
                    },

                    '5': () => {
                        // Scale to (0.7, 1.7) and translate down a little after 0.6s
                        setTimeout(() => {
                            const translateYDistance = 150; // Adjust this value as needed
                            el.style.transition = 'opacity 0.4s ease-out, transform 0.5s ease-out';
                            el.style.opacity = 0;
                            el.style.transform = `translate(${initialX}px, ${initialY + translateYDistance}px) scale(0.5, 2)`;
                        }, 600);
                    },
                    '6': () => {
                        // Animate out after delay
                        setTimeout(() => {
                            el.style.transition = 'opacity 0.4s ease-out';
                            el.style.opacity = '0';
                        }, 450);
                    }
                };
    
                // Get the cursor style from cursor
                const cursorStyle = this.cursor?.cursor.cursor.config.style || '1';
    
                // Apply the appropriate behavior or a default behavior
                (timeoutBehaviors[cursorStyle] || timeoutBehaviors['1'])();
            }
        }
    
        //remove event listeners
        function destroy(container) {
            container.removeEventListener('mousemove', mouseMoveListener);
        }
    
    
        // Iterate through the cursor configurations
        cursorConfigs.forEach(config => {
            if (config.placement === "entire_site") {
                // Apply the effect to the entire body
                new ImageTrail(document.body, config);
            } else if (config.placement === "specific_section") {
                // Apply the effect to specific sections based on the identifier
                const containers = document.querySelectorAll(config.section_identifier);
                containers.forEach(container => {
                    new ImageTrail(container, config);
                });
            }
        });
    
    
        return {destroy};
    }

    if (cursorConfig) {
        const parsedConfig = safeParseCursorConfig(cursorConfig);
        // Ensure it's an array, even if it's a single object
        cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
        
        // Add default placement and section_identifier for preview mode
        cursors = cursors.map(cursor => ({
            cursor: {
                id: cursor.id || null, 
                name: cursor.name || 'Default Cursor',
                type: cursor.type || 'html', 
                hoverSelector: cursor.hoverSelector || 'a', 
                cursor: cursor.cursor || {}, 
                hover: cursor.hover || {} 
            },
            placement: 'entire_site', 
            section_identifier: '', 
            timestamp: Date.now() 
        }));
        console.log("Preview Mode:", cursors);
    } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
        cursors = wpccData.settings;
        console.log("Public Mode:", cursors);
    }

    if (Array.isArray(cursors)) {
        const hephamageConfigs = []; // Array to store Hephamage configurations
        cursors.forEach((cursor) => {
            if (cursor?.cursor?.name == "Hephamage") {
                hephamageConfigs.push(cursor);
            }
        });
        if (hephamageConfigs.length > 0) {
            imageTrailEffect(hephamageConfigs);  // Pass the Hephamage configurations 
        } else {
            console.log("No Hephamage cursors found in the configuration.");
        }
    } else {
        console.warn("No valid cursor settings found.");
    }


  
});