function safeParseCursorConfig(config) {
    try {
        // Attempt to parse as raw JSON first
        return JSON.parse(config);
    } catch {
        try {
            // Attempt to decode and then parse
            return JSON.parse(decodeURIComponent(config));
        } catch (error) {
            console.error('Error decoding cursorConfig:', error);
            return null;
        }
    }
}

document.addEventListener('DOMContentLoaded', () => {
    const urlParams = new URLSearchParams(window.location.search);
    const cursorConfig = urlParams.get('preview_cursor');
    let cursors = []; 

    function hestellipseEffect(cursorConfigs) {
        class DynamicCursor {
            constructor(cursor, container) {
                this.cursor = cursor; // The entire cursor object
                this.container = container;
                this.elements = {}; // To store the created DOM elements
                this.isHovering = false; // To track hover state
                this.wrapper = null; // Wrapper for cursor
                this.points = []; // For SVG animation
                this.segments = 100; // Number of points for the SVG path
                this.mouse = { x: 0, y: 0 }; // Mouse position
                this.handleMouseMove = this.handleMouseMove.bind(this); // Bind mousemove
                this.init();
            }
    
            init() {
                // Create the wrapper
                this.createWrapper();
                this.elements.svg = this.createSVGElement(this.cursor.cursor.cursor.svg, this.cursor.cursor.cursor.path);
    
    
                // Add mousemove event listener to container
                this.container.addEventListener('mousemove', this.handleMouseMove);
    
                // Add hover event listeners
                if (this.cursor.hoverSelector) {
                    const hoverElements = this.container.querySelectorAll(this.cursor.hoverSelector);
                    hoverElements.forEach((el) => {
                        el.addEventListener('mouseenter', this.handleMouseEnter.bind(this));
                        el.addEventListener('mouseleave', this.handleMouseLeave.bind(this));
                    });
                }
    
                this.initSVGAnimation();
                this.resizeSVG();
                window.addEventListener('resize', this.resizeSVG.bind(this));
    
            }
    
            createWrapper() {
                this.wrapper = document.createElement('div');
                this.wrapper.className = `cursor-wrapper ${this.cursor.cursor.name.toLowerCase()}`;
                this.wrapper.style.position = 'absolute'; //  absolute
                this.wrapper.style.top = '0'; // position relative to container
                this.wrapper.style.left = '0';
                this.container.style.position = 'relative';
                this.container.appendChild(this.wrapper);
            }
    
            createSVGElement(svgStyles, pathStyles) {
                const svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                //svg.className = 'custom-cursor-svg';
    
                // Apply SVG attributes
                Object.entries(svgStyles).forEach(([key, value]) => svg.setAttribute(key, value));
                svg.style.position = 'absolute';
                svg.style.pointerEvents = 'none';
                svg.style.top = 0;
                svg.style.left = 0;
    
                const path = document.createElementNS('http://www.w3.org/2000/svg', 'path');
    
                function camelToKebab(key) {
                    return key.replace(/[A-Z]/g, (match) => `-${match.toLowerCase()}`);
                }
    
    
                Object.entries(pathStyles).forEach(([key, value]) => {
                    const kebabKey = camelToKebab(key); // Convert to kebab-case
                    path.setAttribute(kebabKey, value);
                });
    
                svg.appendChild(path);
                this.wrapper.appendChild(svg);
    
                // Store path reference for animation
                this.elements.path = path;
    
                return svg;
            }
    
            initSVGAnimation() {
                if (this.points.length === 0) {
                    for (let i = 0; i < this.segments; i++) {
                        this.points.push({
                            x: this.mouse.x,
                            y: this.mouse.y
                        });
                    }
                }
    
                const animate = () => {
                    let {
                        x: px,
                        y: py
                    } = this.mouse;
    
                    this.points.forEach((point, index) => {
                        point.x = px;
                        point.y = py;
    
                        const next = this.points[index + 1];
                        if (next) {
                            px = px - (point.x - next.x) * 0.6;
                            py = py - (point.y - next.y) * 0.6;
                        }
                    });
    
                    const path = this.elements.path;
                    path.setAttribute('d', `M ${this.points.map((p) => `${p.x} ${p.y}`).join(' L ')}`);
                    requestAnimationFrame(animate);
                };
    
                animate();
            }
    
            resizeSVG() {
                if (this.elements.svg) {
                    const {
                        width: ww,
                        height: wh
                    } = this.container.getBoundingClientRect();
                    const svg = this.elements.svg;
    
                    svg.style.width = `${ww}px`;
                    svg.style.height = `${wh}px`;
                    svg.setAttribute('viewBox', `0 0 ${ww} ${wh}`);
                }
            }
    
            handleMouseMove(event) {
                const rect = this.container.getBoundingClientRect();
                this.mouse.x = event.clientX - rect.left;
                this.mouse.y = event.clientY - rect.top;
            }
    
            handleMouseEnter() {
                this.isHovering = true;
                this.updateStyles();
            }
    
            handleMouseLeave() {
                this.isHovering = false;
                this.updateStyles();
            }
    
            updateStyles() {
    
            }
    
            destroy() {
                // Remove all cursor elements
                if (this.wrapper) {
                    while (this.wrapper.firstChild) {
                        this.wrapper.removeChild(this.wrapper.firstChild);
                    }
                    this.wrapper.remove();
                }
    
                // Remove event listeners from the container
                this.container.removeEventListener('mousemove', this.handleMouseMove);
                if (this.cursor.hoverSelector) {
                    const hoverElements = this.container.querySelectorAll(this.cursor.hoverSelector);
                    hoverElements.forEach((el) => {
                        el.removeEventListener('mouseenter', this.handleMouseEnter);
                        el.removeEventListener('mouseleave', this.handleMouseLeave);
                    });
                }
                window.removeEventListener('resize', this.resizeSVG);
            }
        }
    
        // Iterate through the cursor configurations
        cursorConfigs.forEach(config => {
            if (config.placement === "entire_site") {
                // Apply the effect to the entire body
                new DynamicCursor(config, document.body);
            } else if (config.placement === "specific_section") {
                // Apply the effect to specific sections based on the identifier
                const containers = document.querySelectorAll(config.section_identifier);
                containers.forEach(container => {
                    new DynamicCursor(config, container);
                });
            }
        });
    
    }

    if (cursorConfig) {
        const parsedConfig = safeParseCursorConfig(cursorConfig);
        // Ensure it's an array, even if it's a single object
        cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
        
        // Add default placement and section_identifier for preview mode
        cursors = cursors.map(cursor => ({
            cursor: {
                id: cursor.id || null, 
                name: cursor.name || 'Default Cursor',
                type: cursor.type || 'html', 
                hoverSelector: cursor.hoverSelector || 'a', 
                cursor: cursor.cursor || {}, 
                hover: cursor.hover || {} 
            },
            placement: 'entire_site', 
            section_identifier: '', 
            timestamp: Date.now() 
        }));
    } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
        cursors = wpccData.settings;
    }

    if (Array.isArray(cursors)) {
        const hestellipseConfigs = []; // Array to store Hestellipse configurations
        cursors.forEach((cursor) => {
            if (cursor?.cursor?.name == "Hestellipse") {
                hestellipseConfigs.push(cursor);
            }
        });
        if (hestellipseConfigs.length > 0) {
            hestellipseEffect(hestellipseConfigs); // Pass the Hestellipse configurations 
        } else {
            console.log("No Hestellipse cursors found in the configuration.");
        }
    }
});