function safeParseCursorConfig(config) {
    try {
        // Attempt to parse as raw JSON first
        return JSON.parse(config);
    } catch {
        try {
            // Attempt to decode and then parse
            return JSON.parse(decodeURIComponent(config));
        } catch (error) {
            console.error('Error decoding cursorConfig:', error);
            return null;
        }
    }
}

document.addEventListener('DOMContentLoaded', () => {
    const urlParams = new URLSearchParams(window.location.search);
    const cursorConfig = urlParams.get('preview_cursor');
    let cursors = [];

    class CustomCursor {
        constructor(cursorObject) {
            this.cursorObject = cursorObject;
            this.targetElement = null; // To store the target element
    
            // Create wrapper, cursor elements
            this.cursorWrapper = document.createElement('div');
            this.cursorInner = document.createElement('div');
            this.cursorOuter = document.createElement('div');
    
            this.init();
        }
    
        // Initialize cursor elements and add event listeners
        init() {
            // Function to set the cursor style based on showDefaultCursor
            const setCursorStyle = (element, showDefaultCursor, cursorType = 'default') => {
                element.style.cursor = showDefaultCursor ? cursorType : 'none';
            };
    
            // Determine target element
            if (this.cursorObject.placement === 'specific_section' && this.cursorObject.section_identifier) {
                this.targetElement = document.querySelector(this.cursorObject.section_identifier);
                if (!this.targetElement) {
                    console.warn(`CustomCursor: Target element "${this.cursorObject.section_identifier}" not found. Falling back to document.body.`);
                    this.targetElement = document.body;
                }
    
                // Set showDefaultCursor for specific sections
                const showDefaultCursor = String(this.cursorObject.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                setCursorStyle(this.targetElement, showDefaultCursor);
    
                this.targetElement.addEventListener('mouseenter', this.showCursor.bind(this));
                this.targetElement.addEventListener('mouseleave', this.hideCursor.bind(this));
            } else {
                this.targetElement = document.body;
    
                // Set showDefaultCursor for entire site
                const showDefaultCursor = String(this.cursorObject.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                setCursorStyle(this.targetElement, showDefaultCursor);
            }
    
            // Set up wrapper
            this.cursorWrapper.className = 'cursor-wrapper';
            this.cursorWrapper.style.position = 'fixed';
            this.cursorWrapper.style.top = '0';
            this.cursorWrapper.style.left = '0';
            this.cursorWrapper.style.pointerEvents = 'none';
            this.cursorWrapper.style.willChange = 'transform';
            this.cursorWrapper.style.zIndex = this.cursorObject.cursor.cursor?.options?.zIndex || '99';
    
            // Apply initial mix blend mode to the cursor wrapper
            this.applyMixBlendMode(this.cursorWrapper, this.cursorObject.cursor.cursor?.options?.mixBlendMode);
    
            // Set up cursor elements
            this.cursorInner.className = 'cursor-inner';
            this.cursorOuter.className = 'cursor-outer';
    
            // Apply base styles to cursor elements
            this.applyBaseStyles(this.cursorInner);
            this.applyBaseStyles(this.cursorOuter);
    
            // Apply additional styles for .cursor-inner
            this.cursorInner.style.display = 'flex';
            this.cursorInner.style.alignItems = 'center';
            this.cursorInner.style.justifyContent = 'center';
    
            // Append cursor elements to the wrapper
            this.cursorWrapper.appendChild(this.cursorInner);
            this.cursorWrapper.appendChild(this.cursorOuter);
    
            // Append wrapper to the target element
            this.targetElement.appendChild(this.cursorWrapper);
    
            // Apply initial styles
            this.applyStyles(this.cursorInner, this.cursorObject.cursor.cursor.inner);
            this.applyStyles(this.cursorOuter, this.cursorObject.cursor.cursor.outer);
    
            // Attach mousemove event for cursor position updates on the target element
            this.targetElement.addEventListener('mousemove', this.updateCursorPosition.bind(this));
    
            // Attach hover effects
            const hoverSelector = this.cursorObject.cursor.hover?.options?.hoverSelector || 'a'; // Use targetElement
            this.addHoverListeners(hoverSelector);
    
            // Hide initially if specific section
            if (this.cursorObject.placement === 'specific_section') {
                this.hideCursor();
            }
        }
    
        // Apply base styles to cursor elements
        applyBaseStyles(element) {
            element.style.position = 'absolute'; // Use absolute positioning inside the wrapper
            element.style.top = '0';
            element.style.left = '0';
            element.style.pointerEvents = 'none';
            element.style.willChange = 'transform';
        }
    
        // Apply styles to an element
        applyStyles(element, styles) {
            for (const [key, value] of Object.entries(styles)) {
                if (key === 'backdropFilter') {
                    // Apply both standard and prefixed backdrop-filter
                    element.style.backdropFilter = value;
                    element.style.webkitBackdropFilter = value;
                } else if (key !== 'text') {
                    element.style[key] = value;
                }
            }
            if (styles.text) {
                element.textContent = styles.text;
            }
        }
    
        // Update cursor position based on mouse movement
        updateCursorPosition(event) {
            const { clientX: x, clientY: y } = event;
    
            // Get the computed width and height of the cursor elements
            const outerWidth = parseFloat(window.getComputedStyle(this.cursorOuter).width);
            const outerHeight = parseFloat(window.getComputedStyle(this.cursorOuter).height);
            const innerWidth = parseFloat(window.getComputedStyle(this.cursorInner).width);
            const innerHeight = parseFloat(window.getComputedStyle(this.cursorInner).height);
    
            // Update cursor positions
            this.cursorOuter.style.transform = `translate(${x - outerWidth / 2}px, ${y - outerHeight / 2}px)`;
            this.cursorInner.style.transform = `translate(${x - innerWidth / 2}px, ${y - innerHeight / 2}px)`;
        }
    
        // Apply mix-blend-mode to the cursor wrapper
        applyMixBlendMode(element, mixBlendMode) {
            element.style.mixBlendMode = mixBlendMode || 'normal';
        }
    
        // Add hover event listeners to target elements
        addHoverListeners(hoverSelector) {
            // Function to set the cursor style based on showDefaultCursor
            const setCursorStyle = (element, showDefaultCursor, cursorType = 'default') => {
                element.style.cursor = showDefaultCursor ? cursorType : 'none';
            };
    
            const hoverTargets = this.targetElement.querySelectorAll(hoverSelector);  // Use targetElement
    
            hoverTargets.forEach((target) => {
                target.addEventListener('mouseenter', () => {
                    const showDefaultHoverCursor = String(this.cursorObject.cursor.hover?.options?.showDefaultCursor).toLowerCase() === 'true';
                    setCursorStyle(target, showDefaultHoverCursor, 'pointer');
                    this.applyStyles(this.cursorInner, this.cursorObject.cursor.hover.inner);
                    this.applyStyles(this.cursorOuter, this.cursorObject.cursor.hover.outer);
    
                    // Apply hover mix blend mode to the cursor wrapper
                    this.applyMixBlendMode(this.cursorWrapper, this.cursorObject.cursor.hover?.options?.mixBlendMode);
                });
    
                target.addEventListener('mouseleave', () => {
                    const showDefaultCursor = String(this.cursorObject.cursor.cursor?.options?.showDefaultCursor).toLowerCase() === 'true';
                    setCursorStyle(target, showDefaultCursor);
                    this.applyStyles(this.cursorInner, this.cursorObject.cursor.cursor.inner);
                    this.applyStyles(this.cursorOuter, this.cursorObject.cursor.cursor.outer);
    
                    // Revert to initial mix blend mode on mouse leave
                    this.applyMixBlendMode(this.cursorWrapper, this.cursorObject.cursor.cursor?.options?.mixBlendMode);
                });
            });
        }
    
        showCursor() {
            this.cursorWrapper.style.display = 'block'; // Or visibility: 'visible';
        }
    
        hideCursor() {
            this.cursorWrapper.style.display = 'none'; // Or visibility: 'hidden';
        }
    }

    if (cursorConfig) {
        const parsedConfig = safeParseCursorConfig(cursorConfig);
        // Ensure it's an array, even if it's a single object
        cursors = Array.isArray(parsedConfig) ? parsedConfig : [parsedConfig];
        
        // Add default placement and section_identifier for preview mode
        cursors = cursors.map(cursor => ({
            cursor: {
                id: cursor.id || null, 
                name: cursor.name || 'Default Cursor',
                type: cursor.type || 'html', 
                hoverSelector: cursor.hoverSelector || 'a', 
                cursor: cursor.cursor || {}, 
                hover: cursor.hover || {} 
            },
            placement: 'entire_site', 
            section_identifier: '', 
            timestamp: Date.now() 
        }));
    } else if (typeof wpccData !== 'undefined' && Array.isArray(wpccData.settings)) {
        cursors = wpccData.settings;
    }

    if (Array.isArray(cursors)) {
        const persephontConfigs = []; // Array to store Persephont configurations
        cursors.forEach((cursor) => {
            if (cursor?.cursor?.name == "Persephont") {
                persephontConfigs.push(cursor);
            }
        });
        if (persephontConfigs.length > 0) {
            persephontConfigs.forEach(config => {
                new CustomCursor(config); // Create a new instance for each config
            });
        } else {
            console.log("No Persephont cursors found in the configuration.");
        }
    }
});