import { useState, useEffect } from '@wordpress/element';
import React from '@wordpress/element';
import axios from 'axios';

const ActivationBlock = () => {
    const [code, setCode] = useState('');
    const [loading, setLoading] = useState(false);
    const [message, setMessage] = useState('');
    const [isActivated, setIsActivated] = useState(
        window?.pluginActivationData?.activationStatus || false
    );

    // Safe access to window data
    const data = window?.pluginActivationData || {};
    const { translations = {}, activationRestUrl, nonce } = data;

    // Cleanup function for component unmount
    useEffect(() => {
        return () => {
            setLoading(false);
            setMessage('');
        };
    }, []);

    // Input validation
    const isValidCode = (code) => {
        return /^([a-f0-9]{8})-(([a-f0-9]{4})-){3}([a-f0-9]{12})$/i.test(code.trim());
    };

    const handleSubmit = async (e) => {
        e.preventDefault();
        
        if (!code.trim() || !activationRestUrl || !nonce) {
            setMessage('invalidCode');
            return;
        }

        setLoading(true);
        setMessage('');

        try {
            const response = await axios.post(activationRestUrl, 
                { purchase_code: code.trim() },
                {
                    headers: {
                        'X-WP-Nonce': nonce,
                        'Content-Type': 'application/json',
                    },
                    timeout: 10000, // 10 second timeout
                }
            );

            if (response?.data?.success) {
                setIsActivated(true);
                setMessage('activationSuccess');
                window.pluginActivationData.activationStatus = true;
            } else {
                setMessage('invalidCode');
            }
        } catch (error) {
            console.error('Activation error:', error);
            if (error?.response?.data?.message) {
                setMessage(error.response.data.message);
            } else if (error?.code === 'ECONNABORTED') {
                setMessage('timeoutError');
            } else {
                setMessage('serverError');
            }
        } finally {
            setLoading(false);
        }
    };

    if (!translations || !activationRestUrl || !nonce) {
        return (
            <div className="wrap">
                <div className="notice notice-error">
                    <p>Configuration error. Please refresh the page.</p>
                </div>
            </div>
        );
    }

    return (
        <div className="wrap">
            <h1>{translations.pluginActivation || 'Plugin Activation'}</h1>
            {isActivated ? (
                <div className="notice notice-success">
                    <p>{translations.activatedMessage}</p>
                </div>
            ) : (
                <form onSubmit={handleSubmit} aria-label="Plugin activation form">
                    <div className="form-field">
                        <label htmlFor="purchase-code" className="screen-reader-text">
                            {translations.enterCode}
                        </label>
                        <input
                            id="purchase-code"
                            type="text"
                            value={code}
                            onChange={(e) => setCode(e.target.value)}
                            placeholder={translations.enterCode}
                            className="regular-text"
                            style={{ width: '400px', maxWidth: '100%' }}
                            disabled={loading}
                            aria-invalid={message === 'invalidCode'}
                            aria-describedby={message ? "activation-message" : undefined}
                        />
                    </div>
                    <button
                        type="submit"
                        className="button button-primary"
                        disabled={loading || !code.trim()}
                        aria-busy={loading}
                    >
                        {loading ? translations.activating : translations.activate}
                    </button>
                    {message && (
                        <p 
                            id="activation-message"
                            className={`notice ${
                                message === 'activationSuccess' 
                                    ? 'notice-success' 
                                    : 'notice-error'
                            }`}
                            style={{ marginTop: '15px' }}
                        >
                            {translations[message]}
                        </p>
                    )}
                </form>
            )}
            <p className="description" style={{ marginTop: '20px' }}>
                {translations.findCodeHelp}
            </p>
        </div>
    );
};

export default ActivationBlock;