// import React from '@wordpress/element';
import * as React from 'react';
import { __ } from '@wordpress/i18n';
import { useState } from 'react';
import { Icon, close } from '@wordpress/icons';
import { CursorData, CursorProperties } from "./CursorList" //relative path

interface SavedCursorListProps {
    onAddCursorClick: () => void; // Function that takes no arguments and returns nothing (void)
    cursors: CursorData[];        // Assuming you have a CursorData interface defined
    setSavedCursors: (cursors: CursorData[]) => void; // Function that takes an array of CursorData and returns void
    restUrl: string;            // URL, so it should be a string
}

const SavedCursorList: React.FC<SavedCursorListProps> = ({ onAddCursorClick, cursors, setSavedCursors, restUrl }) => {
    const [isDeleting, setIsDeleting] = useState(false);
    const [deletingCursorId, setDeletingCursorId] = useState<number | null>(null);
    const [isEditModalOpen, setIsEditModalOpen] = useState(false);
    const [editingCursor, setEditingCursor] = useState<CursorData | null>(null);
    const [activeTab, setActiveTab] = useState<"general" | "cursor" | "hover">("general");


    const handleDeleteCursor = async (cursorId: number) => {
        if (isDeleting) return;
    
        setIsDeleting(true);
        setDeletingCursorId(cursorId);
    
        try {
            const response = await fetch(`${restUrl}/cursor-settings/${cursorId}`, {
                method: 'DELETE',
                headers: {
                    'Content-Type': 'application/json',
                },
            });
    
            if (!response.ok) {
                const errorData = await response.json();
                throw new Error(errorData.message || 'Failed to delete cursor');
            }
    
            // Ensure you're updating state properly
            setSavedCursors((prevCursors: CursorData[]) => prevCursors.filter((cursor:CursorData) => cursor.cursor.id !== cursorId));
    
        } catch (error) { 
            console.error("Error deleting cursor:", error);
            alert("An error occurred while deleting the cursor.");
        } finally {
            setIsDeleting(false);
            setDeletingCursorId(null);
        }
    };

    const handleEditCursor = (cursor:CursorData) => {
      
      setEditingCursor(cursor);
      console.log(editingCursor)
      setIsEditModalOpen(true);
    };

    const handleSaveCursor = async () => {
      if (!editingCursor?.cursor.id) return;
  
      // Find the original cursor to retain untouched properties
      const originalCursor = cursors.find(cursor => cursor.cursor.id === editingCursor.cursor.id);
      if (!originalCursor) {
        alert("Cursor not found!");
        return;
      }
  
      // Deep merge function to handle nested objects
      function deepMerge(target: any, source: any) {
        for (const key in source) {
          if (typeof source[key] === 'object' && source[key] !== null && !Array.isArray(source[key])) {
            if (!target[key] || typeof target[key] !== 'object') {
              target[key] = {};
            }
            deepMerge(target[key], source[key]);
          } else {
            target[key] = source[key];
          }
        }
        return target;
      }
  
      // Start with a copy of the original cursor
      const updatedCursor = JSON.parse(JSON.stringify(originalCursor));
  
      // Merge the editingCursor data deeply into the copy
      deepMerge(updatedCursor, editingCursor);
  
  
      try {
        const response = await fetch(`${restUrl}/cursor-settings/${editingCursor.cursor.id}`, {
          method: "PUT",
          headers: {
            "Content-Type": "application/json",
          },
          body: JSON.stringify(updatedCursor),
        });
  
        if (!response.ok) {
          const errorData = await response.json();
          throw new Error(errorData.message || "Failed to save cursor");
        }
  
        // Update local state
        const updatedCursors = cursors.map(cursor =>
          cursor.cursor.id === updatedCursor.cursor.id ? updatedCursor : cursor
        );
        setSavedCursors(updatedCursors);
  
        alert("Cursor updated successfully!");
        setIsEditModalOpen(false);
      } catch (error) {
        console.error("Error updating cursor:", error);
        alert("An error occurred while saving the cursor.");
      }
    };

    const handleChange = (field: string, value: string | number) => {
      setEditingCursor(prev => {
        return (
        {
        ...prev!, // Non-null assertion, since editingCursor should not be null when the modal is open
        [field]: value
      })
            });
    };
  
  const handleNestedChange = (tab: "cursor" | "hover", section: "outer" | "inner" | "options", key: string, value: string | number | boolean) => {
    setEditingCursor(prev => {
      if (!prev) return prev; // Handle potential null scenario

      return {
        ...prev,
        cursor: {
          ...prev.cursor,
          [tab]: {
            ...prev.cursor[tab],
            [section]: {
              ...prev.cursor[tab][section],
              [key]: typeof value === 'string' && !isNaN(Number(value)) ? Number(value) : value  // Convert string to number if applicable
            }
          }
        }
      };
    });
  };

    return (
        <div className="cursor-list">
            <div className="flex items-center justify-between my-2">
              <div>
                <h2 className='text-2xl'>Saved Cursors</h2>
              </div>
              <div>
                <button onClick={onAddCursorClick} className="bg-blue-600 text-white px-4 py-2 rounded-md">{__('Add New Cursor', 'wpcustom-cursors')}</button>
              </div>
            </div>
            {/* Show a message if there are no cursors */}
            {cursors.length === 0 ? (
                <p>No cursors saved yet. Please add one!</p>
            ) : (
                <div>
                    {cursors.map((cursor: CursorData, index) => (
                        <div key={index} className="cursor-item p-4 mb-4 border rounded-lg shadow">
                            <div className="cursor-details mb-2">
                                <h3 className="text-xl">{cursor.cursor.name}</h3>
                                <p>
                                  <strong>Added to:</strong>{" "}
                                  {cursor.placement === "entire_site" ? (
                                    "Entire Site"
                                  ) : cursor.placement === "specific_section" && cursor.section_identifier ? (
                                    cursor.section_identifier
                                  ) : (
                                    "Unknown" // Or some other appropriate fallback
                                  )}
                                </p>
                            </div>

                            {/* Edit and Delete buttons */}
                            <div className="cursor-actions flex space-x-4">
                                <button
                                    className="edit-button bg-blue-500 text-white py-2 px-4 rounded"
                                    onClick={() => handleEditCursor(cursor)}
                                >
                                    Edit
                                </button>
                                <button
                                    className={`delete-button bg-red-500 text-white py-2 px-4 rounded ${isDeleting && cursor.cursor.id === deletingCursorId ? 'opacity-50 cursor-not-allowed' : ''}`}
                                    onClick={() => handleDeleteCursor(cursor.cursor.id)}
                                    disabled={isDeleting && cursor.cursor.id === deletingCursorId}
                                >
                                    {isDeleting && cursor.cursor.id === deletingCursorId ? 'Deleting...' : 'Delete'}
                                </button>
                            </div>
                        </div>
                    ))}
                </div>
            )}

            {isEditModalOpen && (
              <div className="modal z-[100] fixed inset-0 flex justify-center items-center bg-black bg-opacity-50">
                <div className="modal-content bg-white rounded-lg shadow-lg max-h-[90vh] min-w-[500px] flex flex-col">
                  {/* Header */}
                  <div className="modal-header p-4 flex flex-col">
                    <div className="flex justify-between items-center">
                      <h3 className="text-lg">Edit Cursor</h3>
                      <button className="text-gray-500 hover:text-gray-800" onClick={() => setIsEditModalOpen(false)}>
                      <Icon icon={close} size={24} />
                      </button>
                    </div>

                    {/* Tabs */}
                    <div className="tabs mt-4 border-b flex">
                      <button
                        className={`px-4 py-2 ${activeTab === "general" ? "border-b-2 border-blue-500 font-bold" : ""}`}
                        onClick={() => setActiveTab("general")}
                      >
                        General Info
                      </button>
                      <button
                        className={`px-4 py-2 ${activeTab === "cursor" ? "border-b-2 border-blue-500 font-bold" : ""}`}
                        onClick={() => setActiveTab("cursor")}
                      >
                        Cursor Styles
                      </button>
                      <button
                        className={`px-4 py-2 ${activeTab === "hover" ? "border-b-2 border-blue-500 font-bold" : ""}`}
                        onClick={() => setActiveTab("hover")}
                      >
                        Hover Styles
                      </button>
                    </div>
                  </div>

                  {/* Body */}
                  <div className="modal-body flex-grow overflow-y-auto p-4">
                    {activeTab === "general" && (
                      <>
                        {["placement", "section_identifier", "timestamp"].map((key) => (
                          <div key={key} className="flex gap-3 items-center mb-3">
                            <label htmlFor={key}>{key.replace(/_/g, " ")}:</label>
                            <input
                              id={key}
                              type="text"
                              value={(editingCursor[key as keyof CursorData] as string) || ""} // Use empty string as default
                              onChange={(e) => handleChange(key, e.target.value)}
                              className="flex-grow border p-2 rounded"
                            />
                          </div>
                        ))}
                      </>
                    )}

                    {activeTab === "cursor" && (
                      <>
                        {/* Outer Cursor Properties */}
                        {editingCursor?.cursor?.cursor?.outer ? (
                          <>
                            <h4 className="font-bold mb-1 mt-3">Outer Properties</h4>
                            <div className="bg-neutral-100 p-4 rounded-lg">
                              {Object.entries(editingCursor.cursor.cursor.outer).map(([key, value]) => (
                                <div key={`cursor-outer-${key}`} className="flex gap-3 items-center mb-3">
                                  <label htmlFor={`cursor-outer-${key}`}>{key.replace(/_/g, " ")}:</label>
                                  <input
                                    id={`cursor-outer-${key}`}
                                    type="text"
                                    value={value !== null && value !== undefined ? value.toString() : ""} //convert value to string
                                    onChange={(e) => handleNestedChange("cursor", "outer", key, e.target.value)}
                                    className="flex-grow border p-2 rounded"
                                  />
                                </div>
                              ))}
                            </div>
                          </>
                        ) : null }

                        {/* Inner Cursor Properties */}
                        {editingCursor?.cursor?.cursor?.inner ? (
                          <>
                            <h4 className="font-bold mt-2">Inner Properties</h4>
                            <div className="bg-neutral-100 p-4 rounded-lg">
                              {Object.entries(editingCursor.cursor.cursor.inner).map(([key, value]) => (
                                <div key={`cursor-inner-${key}`} className="flex gap-3 items-center mb-3">
                                  <label htmlFor={`cursor-inner-${key}`}>{key.replace(/_/g, " ")}:</label>
                                  <input
                                    id={`cursor-inner-${key}`}
                                    type="text"
                                    value={value !== null && value !== undefined ? value.toString() : ""} //convert value to string
                                    onChange={(e) => handleNestedChange("cursor", "inner", key, e.target.value)}
                                    className="flex-grow border p-2 rounded"
                                  />
                                </div>
                              ))}
                            </div>
                          </>
                        ) : null }

                        {/* Cursor Options */}
                        {editingCursor?.cursor?.cursor?.options ? (
                          <>
                            <h4 className="font-bold mb-1 mt-3">Options</h4>
                            <div className="bg-neutral-100 p-4 rounded-lg">
                              {Object.entries(editingCursor.cursor.cursor.options).map(([key, value]) => (
                                <div key={`cursor-options-${key}`} className="flex gap-3 items-center mb-3">
                                  <label htmlFor={`cursor-options-${key}`}>{key.replace(/_/g, " ")}:</label>
                                  {typeof value === "boolean" ? (
                                    <input
                                      id={`cursor-options-${key}`}
                                      type="checkbox"
                                      checked={value}
                                      onChange={(e) => handleNestedChange("cursor", "options", key, e.target.checked)}
                                    />
                                  ) : (
                                    <input
                                      id={`cursor-options-${key}`}
                                      type="text"
                                      value={value !== null && value !== undefined ? value.toString() : ""} //convert value to string
                                      onChange={(e) => handleNestedChange("cursor", "options", key, e.target.value)}
                                      className="flex-grow border p-2 rounded"
                                    />
                                  )}
                                </div>
                              ))}
                            </div>
                          </>
                        ) : null }
                      </>
                    )}

                    {activeTab === "hover" && (
                      <>
                        {/* Outer Hover Properties */}
                        {editingCursor?.cursor?.hover?.outer ? (
                          <>
                            <h4 className="font-bold mb-1 mt-3">Outer Properties</h4>
                            <div className="bg-neutral-100 p-4 rounded-lg">
                              {Object.entries(editingCursor.cursor.hover.outer).map(([key, value]) => (
                                <div key={`hover-outer-${key}`} className="flex gap-3 items-center mb-3">
                                  <label htmlFor={`hover-outer-${key}`}>{key.replace(/_/g, " ")}:</label>
                                  <input
                                    id={`hover-outer-${key}`}
                                    type="text"
                                    value={value !== null && value !== undefined ? value.toString() : ""} //convert value to string
                                    onChange={(e) => handleNestedChange("hover", "outer", key, e.target.value)}
                                    className="flex-grow border p-2 rounded"
                                  />
                                </div>
                              ))}
                            </div>
                          </>
                        ) : null }

                        {/* Inner Hover Properties */}
                        {editingCursor?.cursor?.hover?.inner ? (
                          <>
                            <h4 className="font-bold mb-1 mt-3">Inner Properties</h4>
                            <div className="bg-neutral-100 p-4 rounded-lg">
                              {Object.entries(editingCursor.cursor.hover.inner).map(([key, value]) => (
                                <div key={`hover-inner-${key}`} className="flex gap-3 items-center mb-3">
                                  <label htmlFor={`hover-inner-${key}`}>{key.replace(/_/g, " ")}:</label>
                                  <input
                                    id={`hover-inner-${key}`}
                                    type="text"
                                    value={value !== null && value !== undefined ? value.toString() : ""} //convert value to string
                                    onChange={(e) => handleNestedChange("hover", "inner", key, e.target.value)}
                                    className="flex-grow border p-2 rounded"
                                  />
                                </div>
                              ))}
                            </div>
                          </>
                        ) : null }

                        {/* Hover Options */}
                        {editingCursor?.cursor?.hover?.options ? (
                          <>
                            <h4 className="font-bold mb-1 mt-3">Options</h4>
                            <div className="bg-neutral-100 p-4 rounded-lg">
                              {Object.entries(editingCursor.cursor.hover.options).map(([key, value]) => (
                                <div key={`hover-options-${key}`} className="flex gap-3 items-center mb-3">
                                  <label htmlFor={`hover-options-${key}`}>{key.replace(/_/g, " ")}:</label>
                                  {typeof value === "boolean" ? (
                                    <input
                                      id={`hover-options-${key}`}
                                      type="checkbox"
                                      checked={value}
                                      onChange={(e) => handleNestedChange("hover", "options", key, e.target.checked)}
                                    />
                                  ) : (
                                    <input
                                      id={`hover-options-${key}`}
                                      type="text"
                                      value={value !== null && value !== undefined ? value.toString() : ""} //convert value to string
                                      onChange={(e) => handleNestedChange("hover", "options", key, e.target.value)}
                                      className="flex-grow border p-2 rounded"
                                    />
                                  )}
                                </div>
                              ))}
                            </div>
                          </>
                        ) : null }
                      </>
                    )}
                  </div>

                  {/* Footer */}
                  <div className="modal-footer p-4 flex justify-end gap-2">
                    <button className="bg-blue-500 text-white px-4 py-2 rounded hover:bg-blue-600" onClick={handleSaveCursor}>
                      Save
                    </button>
                    <button className="bg-gray-200 px-4 py-2 rounded hover:bg-gray-300" onClick={() => setIsEditModalOpen(false)}>
                      Cancel
                    </button>
                  </div>
                </div>
              </div>
            )}
        </div>
    );
};

export default SavedCursorList;